import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

class JRedGreenButton extends JButton
{	
	JRedGreenButton(String text)
	{
		super(text);
		red();
	}
	
	void red()
	{
		setForeground(Color.RED);
	}
	
	void green()
	{
		setForeground(Color.GREEN.darker().darker());
	}	
}

class MoteCtrl extends JPanel implements ActionListener
{	
	private final WS2 ws2;
	private final int nodeID;
	
	private final JLabel nodeIDLabel;
	private final JFormattedTextField sampleFreqField;
	
	private final JButton setInitValuesButton = new JButton("Set Init Values");
	final JRedGreenButton getResultButton = new JRedGreenButton("Get Result");
	final JRedGreenButton getRawDataButton = new JRedGreenButton("Get Raw Data");
	final JRedGreenButton getPSDataButton = new JRedGreenButton("Get PS Data");
	
	MoteCtrl(WS2 ws2, int nodeID, float sampleFreq)
	{
		this.ws2 = ws2;
		this.nodeID = nodeID;
		
		setLayout(new GridLayout());
		nodeIDLabel = new JLabel(Integer.toHexString(nodeID), SwingConstants.CENTER);
		sampleFreqField = new JFormattedTextField(sampleFreq);
		
		setInitValuesButton.addActionListener(this);
		getResultButton.addActionListener(this);
		getRawDataButton.addActionListener(this);
		getPSDataButton.addActionListener(this);
		
		add(nodeIDLabel);
		add(sampleFreqField);
		add(setInitValuesButton);
		add(getResultButton);
		add(getRawDataButton);
		add(getPSDataButton);
	}
	
	float getSampleFreq() { return (Float)sampleFreqField.getValue(); }
	
	private final Runnable setInitValues = new Runnable()
	{
		public void run()
		{
			float sampleFreq = (Float)sampleFreqField.getValue();
			ws2.setInitValues(nodeID, sampleFreq);			
		};
	};
	
	private final Runnable getResult = new Runnable()
	{
		public void run()
		{
			ws2.requestRootData(nodeID);
		};
	};
	
	private final Runnable getRawData = new Runnable()
	{
		public void run()
		{
			ws2.requestRawData(nodeID);
		};
	};
	
	private final Runnable getPSData = new Runnable()
	{
		public void run()
		{
			ws2.requestPowerSpectrumData(nodeID);
		};
	};

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == setInitValuesButton)
			SwingUtilities.invokeLater(setInitValues);
		else if(e.getSource() == getResultButton)
			SwingUtilities.invokeLater(getResult);
		else if(e.getSource() == getRawDataButton)
			SwingUtilities.invokeLater(getRawData);
		else if(e.getSource() == getPSDataButton)
			SwingUtilities.invokeLater(getPSData);
	}
}
