import java.io.*;
import java.util.*;

class WS2Configuration
{
	double [] W_EH;
	double [] W_AH;
	double [][] W_AD;
	
	double [] start;
	double [] end;
	
	int [] motes;
	Map<Integer, Float> moteFrequencies = new HashMap<Integer, Float>();
	
	int numNodes;
	int numSamples;
	boolean debug;
	
	private static final double [][] resize(double [] m, int rows, int cols)
	{
		assert(m.length == rows * cols);
		
		double [][] resized = new double[rows][cols];
		for(int i = 0; i < rows; i++)
			System.arraycopy(m, i * cols, resized[i], 0, cols);
			
		return resized;
	}
	
	private static int [] getHexArray(Properties props, String key)
	{
		String [] strings = props.getProperty(key).split(",");
		int [] ints = new int[strings.length];
		for(int i = 0; i < ints.length; i++)
			ints[i] = Integer.parseInt(strings[i].trim(), 16);
		
		return ints;
	}
	
	private static double [] getDoubleArray(Properties props, String key)
	{
		String [] strings = props.getProperty(key).split(",");
		double [] doubles = new double[strings.length];
		for(int i = 0; i < doubles.length; i++)
			doubles[i] = Double.parseDouble(strings[i].trim());
		
		return doubles;
	}
	
	WS2Configuration() throws IOException
	{
		Properties props = new Properties();
		props.load(new FileInputStream("ws2.properties"));
		
		numNodes = Integer.parseInt(props.getProperty("numNodes"));

		W_EH = getDoubleArray(props, "W_EH");
		assert(W_EH.length == numNodes);
		W_AH = getDoubleArray(props, "W_AH");
		assert(W_AH.length == numNodes);
		W_AD = resize(getDoubleArray(props, "W_AD"), numNodes, DLAC.NUM_ROOTS);
		
		start = getDoubleArray(props, "start");
		assert(start.length == numNodes);
		end = getDoubleArray(props, "end");
		assert(end.length == numNodes);
		
		motes = getHexArray(props, "motes");
		double [] frequencies = getDoubleArray(props, "frequencies");
		assert(motes.length == frequencies.length);
		for(int i = 0; i < motes.length; i++)
			moteFrequencies.put(motes[i], (float)frequencies[i]);
		
		numSamples = Integer.parseInt(props.getProperty("numSamples"));
		debug = Boolean.parseBoolean(props.getProperty("debug"));
	}
}
