import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

class WS2GUI extends JFrame
{
	private final WS2 ws2;

	private final Map<Integer, MoteCtrl> nodeIDtoPanel = new HashMap<Integer, MoteCtrl>();
	
	WS2GUI(WS2 ws2, WS2Configuration config)
	{
		super("Wireless Sample 2");
		this.ws2 = ws2;
		JPanel contents = new JPanel();
		contents.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setContentPane(contents);
		setLayout(new BorderLayout());
		setResizable(false);
				
		getContentPane().add(makeIndexPanel(config), BorderLayout.NORTH);
		getContentPane().add(makeDLACPanel(config), BorderLayout.CENTER);
		getContentPane().add(makeStartPanel(), BorderLayout.SOUTH);
		
		pack();		
		
		// The following code centers the frame in the center of default
		// screen.
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice gd = ge.getDefaultScreenDevice();
		GraphicsConfiguration gc = gd.getDefaultConfiguration();
		Rectangle sb = gc.getBounds();
		Dimension frameSize = this.getSize();
		setLocation((sb.width - frameSize.width) / 2,
					(sb.height - frameSize.height) / 2);
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
	}
	
	private JPanel makeIndexPanel(WS2Configuration config)
	{
		JPanel freqPanel = new JPanel(new GridLayout(2, DLAC.NUM_ROOTS + 1));
		freqPanel.add(new JLabel("<html><b>Start Freq:</b></html>"));
		for(double i : config.start)
			freqPanel.add(new JLabel("" + i));
		freqPanel.add(new JLabel("<html><b>End Freq:</b></html>"));
		for(double i : config.end)
			freqPanel.add(new JLabel("" + i));

		return freqPanel;
	}
	
	private JPanel makeDLACPanel(WS2Configuration config)
	{
		JPanel dp = new JPanel();
		dp.setLayout(new BoxLayout(dp, BoxLayout.Y_AXIS));
		dp.add(new JSeparator());
		JPanel legend = new JPanel(new GridLayout());
		for(String text : new String [] { "Node ID", "Sampling Frequency", "", "", "", "" })
		{
			JLabel label = new JLabel(text);
			label.setFont(label.getFont().deriveFont(0.9f * label.getFont().getSize()));
			label.setHorizontalAlignment(SwingConstants.CENTER);
			legend.add(label);
		}
			
		dp.add(legend);
		for(int nodeID : config.motes)
		{
			MoteCtrl moteCtrl = new MoteCtrl(ws2, nodeID, config.moteFrequencies.get(nodeID));
			nodeIDtoPanel.put(nodeID, moteCtrl);
			dp.add(moteCtrl);
		}
		
		return dp;
	}
	
	private JPanel makeStartPanel()
	{
		JButton startDLACButton = new JButton("Start DLAC");
		startDLACButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				SwingUtilities.invokeLater(startDLAC);
			}
		});
		JButton saveResultsButton = new JButton("Save Results");
		saveResultsButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				SwingUtilities.invokeLater(saveResults);
			}
		});
		
		JPanel sp = new JPanel();
		sp.add(startDLACButton);
		sp.add(saveResultsButton);		
		return sp;
	}
	
	private Runnable startDLAC = new Runnable()
	{
		public void run()
		{
			ws2.startDLAC();
			JOptionPane.showMessageDialog(WS2GUI.this,
				"Please wait for the green LED to light up.",
			    "Sampling in Progress",
			    JOptionPane.INFORMATION_MESSAGE);
		};
	};
	
	private Runnable saveResults = new Runnable()
	{
		public void run()
		{
			for(int nodeID : nodeIDtoPanel.keySet())
			{
				float sampleFreq = nodeIDtoPanel.get(nodeID).getSampleFreq();
				
				if(!ws2.saveRoot(nodeID, sampleFreq))
					JOptionPane.showMessageDialog(WS2GUI.this, "Node "
						+ WS2.intToHexString(nodeID) + " is missing root data.",
						"Error", JOptionPane.ERROR_MESSAGE);
					
				if(!ws2.saveRaw(nodeID))
					JOptionPane.showMessageDialog(WS2GUI.this, "Node "
						+ WS2.intToHexString(nodeID) + " is missing raw data.",
						"Error", JOptionPane.ERROR_MESSAGE);
	
				ws2.savePowerSpectrum(nodeID);
				// fail silently since PS data is not critical
			}
		}
	};
	
	void makePSDataButtonGreen(int nodeID)
	{
		if(nodeIDtoPanel.containsKey(nodeID))
			nodeIDtoPanel.get(nodeID).getPSDataButton.green();
	}

	void makeRawDataButtonGreen(int nodeID)
	{
		if(nodeIDtoPanel.containsKey(nodeID))
			nodeIDtoPanel.get(nodeID).getRawDataButton.green();
	}

	void makeResultButtonGreen(int nodeID)
	{
		if(nodeIDtoPanel.containsKey(nodeID))
			nodeIDtoPanel.get(nodeID).getResultButton.green();
	}
}
