#ifndef _WS_CONST_
#define _WS_CONST_

/******************************************************************************
 *
 * number of raw readings in one axis
 *
 ******************************************************************************/
#define NUM_SAMPLES 2048
#define SAMPLE_TIMEOUT 1024*5  // we are sampling 2048 samples @ ~560Hz, making 5s enough
#define NUM_BYTES	64+(6*NUM_SAMPLES)
#define HALF_LENGTH	NUM_SAMPLES/2
	
#define NUM_ROOTS   5  // Number of natural frequencies (peaks/roots) to find
#define FLOATING_MSG_SIZE NUM_ROOTS*5		// The number of floating point numbers to send

#if defined(PLATFORM_PC)
#define BS_ADDR TOS_UART_ADDR
#else
#define BS_ADDR 0
#endif

TOSH_ASSIGN_PIN(GPIO_93, A, 93);

typedef struct RawMsg {
	uint16_t id;
	uint8_t data[NUM_BYTES];
} __attribute((packed)) RawMsg;

typedef struct GetRawMsg {
	uint16_t id;
} __attribute((packed)) GetRawMsg;

typedef struct AccelResetMsg {
    /* Empty payload! */
} __attribute((packed)) AccelResetMsg;

typedef struct StartMsg {
    /* Empty payload! */
} __attribute((packed)) StartMsg;

typedef struct RootMsg {
	uint16_t id;
	uint16_t numAvg;
	float data[FLOATING_MSG_SIZE];
} __attribute((packed)) RootMsg;

typedef struct GetRootMsg {
	uint16_t id;
} __attribute((packed)) GetRootMsg;

typedef struct SetSamplingFreqMsg {
	uint16_t id;			// The ID of the destination mote.
	uint16_t st[5];
	uint16_t sp[5];
	float samplingFreq;
} __attribute((packed)) SetSamplingFreqMsg;

typedef struct PSMsg {
	uint16_t id;
	float data[FLOATING_MSG_SIZE];
} __attribute((packed)) PSMsg;

typedef struct GetPSMsg {
	uint16_t id;
	uint16_t index;		// the index of the first Power Spectrum variable in the PS array.
} __attribute((packed)) GetPSMsg;

/**
 * This message is sent whenever a mote fails to obtain sensor data.
 */
typedef struct ErrorMsg {
	uint16_t id;
} __attribute((packed)) ErrorMsg;

enum {
  AM_RAWMSG = 10,
  AM_GETRAWMSG = 64,
  AM_ACCELRESETMSG = 32,
  AM_STARTMSG = 96,
  AM_ROOTMSG = 100,
  AM_GETROOTMSG = 101,
  AM_SETSAMPLINGFREQMSG = 102,
  AM_GETPSMSG = 103,
  AM_PSMSG = 104,
  AM_ERRORMSG = 105,
};	
	
#endif
