package servilla;

import java.util.Vector;
import org.apache.commons.lang.ArrayUtils;
import servilla.util.*;

/**
 * This is the script's stack.
 * 
 * The script's stack is an array of bytes.
 * 
 * @author Chien-Liang Fok
 */
public class Stack {
	/**
	 * Holds the stack's data.
	 */
	private Vector<Byte> stack = new Vector<Byte>();
	
	/**
	 * The constructor, creates an empty stack.
	 *
	 */
	public Stack() { }
	
	/**
	 * Creates a stack of a particular size.  The stack is initalized with all zeros.
	 * @param size
	 */
	public Stack(int size) { 
		stack = new Vector<Byte>(size); 
		for (int i = 0; i < size; i++) {
			stack.add(new Byte((byte)0));
		}
	}
	
	/**
	 * Returns the stack pointer.  The stack pointer is always equal to
	 * the size of the stack.
	 * 
	 * @return the stack pointer.
	 */
	public int sp() { return stack.size();}
	
	/**
	 * Adds bytes to the top of the stack stack.
	 * 
	 * @param bv The bytes to add.
	 */
	public void addBytes(Vector<Byte> bv) {
		stack.addAll(bv);
	}
	
	public void setBytes(Vector<Byte> bv, int indx) {
		try {
			for (int i = 0; i < bv.size(); i++) {
				stack.setElementAt(bv.get(i), indx+i);
			}
		} catch(Exception e) {
			Debugger.log("Stack", "ERROR: Trying to set bytes in stack at random locations!");
			e.printStackTrace();
		}
	}
	
	/**
	 * Converts the stack into a vector of bytes.
	 * 
	 * @return a vector of bytes consisting of the stack's state.
	 */
	public Vector<Byte> toByteVector() { return stack; }
	
	/**
	 * Returns a string representation of this class.
	 * 
	 * @return a string representation of this class.
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("Stack: sp="+sp()+": ");
		sb.append(ArrayToHexString.toString(ArrayUtils.toPrimitive(stack.toArray(new Byte[]{}))));
		return sb.toString();
	}
	
}
