// $Id: VarUtil.java,v 1.3 2005/11/17 23:24:59 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * VarUtil.java
 *
 * @author Created by Omnicore CodeGuide
 */

package servilla;

import java.util.*;
import servilla.variables.*;
import servilla.service.tree.DataType;

public class VarUtil implements ServillaConstants
{
	/**
	 * Retrieves a variable stored in the supplied byte array.
	 * The byte array must be of the form [vtype], [var], [vtype], [var], ...
	 * startIndex must point to a vtype.
	 * 
	 * @param startIndex The array index where the variable is stored
	 * @param data The data bytes.
	 * @return The variable being pointed to by startIndex
	 */
	public static ServillaVariable getField(int startIndex, short[] data) 
		throws ServillaException 
	{
		short vtype = data[startIndex++];
		if (vtype == DataType.FLOAT32.value()) {
			int i = data[startIndex++];
			i |= data[startIndex++] << 8;
			i |= data[startIndex++] << 16;
			i |= data[startIndex++] << 24;
			float f = Float.intBitsToFloat(i);  // convert to Java float data type
			return new Float32Var(f);
		} else if (vtype == DataType.INT16.value()) {
			int i = data[startIndex++];
			i |= data[startIndex] << 8;
			return new Int16Var((short)i);
		} else if (vtype == DataType.STRING.value()) {
			Vector<Short> v = new Vector<Short>();
			while (data[startIndex] != 0 && startIndex < data.length) {
				v.add(data[startIndex++]);
			}
			byte[] ca = new byte[v.size()];
			for (int i = 0; i < v.size(); i++) {
				ca[i] = (byte)v.get(i).shortValue();
			}
			String s = new String(ca);
			if (startIndex == data.length && data[data.length-1] != 0)
				throw new ServillaException("Non terminated string.");
			return new StringVar(s);
		} else if (vtype == DataType.ANY.value()) {
			return new TypeVar(DataType.ANY);
		} else
			return new InvalidVar();
			
	}
	
	
	public static ServillaVariable pop(int sp, short [] data) {
		short vtype = data[sp--];
		if (vtype == DataType.INT16.value()) {
			short val = data[sp - 1];
			val |= (data[sp] << 8);
			return new Int16Var(val);
		} else if (vtype == DataType.FLOAT32.value()) {
			int floatInt = 0;
			floatInt = data[sp];
			floatInt |= data[sp - 1] << 8;
			floatInt |= data[sp - 2] << 16;
			floatInt |= data[sp - 3] << 24;
			return new Float32Var(Float.intBitsToFloat(floatInt));
		} else if (vtype == DataType.STRING.value()) {
			Vector<Byte> v = new Vector<Byte>();
			int i = 0;
			while (data[sp=i] != 0x00) {
				v.add(new Byte((byte)data[sp-i]));
				i++;
			}
			byte[] bytes = new byte[v.size()];
			for (i = 0; i < bytes.length; i++) {
				bytes[i] = v.get(i).byteValue();
			}
			return new StringVar(new String(bytes));
		} else if (vtype == DataType.TYPE.value()) {
			//return new TypeVar((byte)data[sp]);
			return new TypeVar(DataType.ANY);
		} else
			return new InvalidVar();
		
	}
	
//	private static byte[] save5Bytes(int vtype, int data1, int data2) {
//		byte[] result = new byte[5];
//		result[0] = (byte)(data1 & 0xff);
//		result[1] = (byte)(data1 >> 8);
//		result[2] = (byte)(data2 & 0xff);
//		result[3] = (byte)(data2 >> 8);
//		result[4] = (byte)vtype;
//		return result;
//	}
	
//	private static byte[] save3Bytes(int vtype, int data) {
//		byte[] result = new byte[3];
//		result[0] = (byte)(data & 0xff);
//		result[1] = (byte)(data >> 8);
//		result[2] = (byte)vtype;
//		return result;
//	}
	
	/**
	 * Returns a byte array for storing the variable in an operand stack.
	 * 
	 * @param v The variable to convert into bytes
	 * @return The variable in the form of a byte array
	 */
	public static byte[] toStackBytes(ServillaVariable v) {
		/*short vtype = v.getType();
		switch(vtype) {
		case AGILLA_TYPE_VALUE:
			return save3Bytes(vtype, ((ServillaValue)v).getValue());
		case AGILLA_TYPE_READING:
			ServillaReading r = (ServillaReading)v;
			int rtype = r.getReadingType(), reading = r.getReading();
			return save5Bytes(vtype, rtype, reading);
		case AGILLA_TYPE_STRING:
			return save3Bytes(vtype, ((ServillaString)v).getString());
		case AGILLA_TYPE_TYPE:
			return save3Bytes(vtype, ((ServillaType)v).getDataType());
		case AGILLA_TYPE_RTYPE:
			return save3Bytes(vtype, ((ServillaRType)v).getReadingType());
		case AGILLA_TYPE_AGENTID:
			return save3Bytes(vtype, ((int)v).getID());
		case AGILLA_TYPE_LOCATION:
			return save5Bytes(vtype, ((ServillaLocation)v).getx(),((ServillaLocation)v).gety());
		default:
			return null;		
		}*/
		return null;
	}
}

