package servilla.compiler;

/**
 * A token containing a single integer value. This token is created by the
 * scanner (Yylex), and passed within Symbol objects to the parser 
 * (ServiceSpecParser).
 * 
 * @author Chien-Liang Fok <liangfok@wustl.edu>
 *
 */
public class LongToken extends Token {
    private long longValue;
    
    public LongToken(String text, int lineBegin, int charBegin) {
    	super(lineBegin, charBegin);
    	if (text.startsWith("0x")) {
    		this.longValue = Long.valueOf(text.substring(2), 16);
    	} else
    		this.longValue = new Long(text).longValue();
    }
    
    public long value() {
      return longValue;
    }
    
    public String toString() {
      return super.toString() + ", integer: " + longValue;
    }
}