// $Id: AgentInjector.java,v 1.10 2005/12/13 00:18:42 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
package servilla.compiler.tree;

/**
 * An assignment statement assigns the results of an expression
 * to a variable.
 * 
 * @author Chien-Liang Fok
 *
 */
public class AssignmentStatement extends Statement {
	private AssignmentExpr expr;
	
	/**
	 * Creates an assignment statement.
	 * 
	 * @param expr The assignment expression.
	 * @param lineno The line number of the program code that this statement maps to.
	 * @param charno The character number of the program code that this statement maps to.
	 */
	public AssignmentStatement(AssignmentExpr expr, int lineno, int charno) {
		super(lineno, charno);
		this.expr = expr;
	}
	
	/**
	 * Generates assembly code for the assignment statement.
	 */
	public void assemble(CodeWriter cw) throws AssemblerException {
		expr.assemble(cw);
		popRetVal(cw, expr, "Discard result of assignment expression.");
	}
	
	public String toString() { return "(AssignmentStatement " + expr + ")"; }

	@Override
	public long initAddr(long startAddr) { return expr.initAddr(startAddr); }

	@Override
	public long localVarSize() { return expr.localVarSize(); }
}
