package servilla.compiler.tree;

/**
 * A call statement is used to make a function call.  It consists of
 * a call expression.
 * 
 * @author Chien-Liang Fok
 */
public class CallStatement extends Statement {
	private CallExpr expr;
	
	public CallStatement(CallExpr expr, int lineno, int charno) {
		super(lineno, charno);
		this.expr = expr;
	}
	
	public void assemble(CodeWriter cw) throws AssemblerException {
		expr.assemble(cw);
		popRetVal(cw, expr, "Discard result of call expression");
	}
	
	public String toString() {
		return "(CALL " + expr + ")";
	}

	@Override
	public long initAddr(long startAddr) {
		return expr.initAddr(startAddr);
	}
	
	@Override
	public long localVarSize() {
		return expr.localVarSize();
	}
}
