package servilla.compiler.tree;

import servilla.service.tree.DataType;

public class FloatDeclExpr extends VarDeclExpr {

	public FloatDeclExpr(String name, int lineno, int charno) {
		this(name, null, lineno, charno);
	}
	
	public FloatDeclExpr(String name, Expr initVal, int lineno, int charno) {
		super(name, DataType.INT, initVal, lineno, charno);
	}

	/**
	 * Returns the number of bytes needed by local variables within the expression.
	 */
	@Override
	public long localVarSize() { return type().size();  }
	
	/**
	 * Assembles the float decl expression. Adds itself to the CodeWriter's list of 
	 * local variables.  If there is an initial value, it creates an assignment 
	 * statement to initialize it.
	 */
	public void assemble(CodeWriter cw) throws AssemblerException {
		cw.addLocalVar(this, lineno, charno);
		Expr initValExpr = initVal();
		if (initValExpr != null) {
			DataType initType = initValExpr.retType(cw);
			if (initType == DataType.FLOAT)
				initVar(cw, new VarRef(name(), lineno, charno), initValExpr);
			else
				throw new AssemblerException("Type mismatch, got " + initType + ", expected " + DataType.FLOAT, lineno, charno);
		}
	}

}
