package servilla.compiler.tree;

import servilla.service.tree.DataType;

public class FloatVal extends Var {
	
	private float fltVal;
	
	public FloatVal(float fltVal, int lineno, int charno) {
		super(lineno, charno);
		this.fltVal = fltVal;
	}
	
	@Override
	public void assemble(CodeWriter cw, AssembleType aType) throws AssemblerException {
		cw.add(new Instruction(ISA.pushFltConst, lineno, charno).setParam(fltVal));
	}

	/**
	 * Returns the number of bytes needed by local variables.  This is the amount of memory
	 * consumed by the value in the heap.
	 */
	@Override
	public long localVarSize() { return DataType.FLOAT.size(); }

	/**
	 * Returns the size of the returned data.  This is the amount of memory consumed
	 * by the value on the stack.
	 * 
	 * @param cw The Code Writer (provides the frame in which the variable is defined).
	 * @return The size of the returned data.
	 * @throws AssemblerException
	 */
	@Override
	public long retSize(CodeWriter cw) throws AssemblerException { return localVarSize(); }


	/**
	 * Returns the type of data that this variable represents.
	 * 
	 * @param cw The Code Writer (provides the frame in which the variable is defined).
	 * @return the type of this variable.
	 */
	@Override
	public DataType retType(CodeWriter cw) throws AssemblerException { return DataType.FLOAT; }

}
