package servilla.compiler.tree;

import servilla.service.tree.*;
import servilla.service.*;

/**
 * A RxnDecl is a declaration of a reaction specification.  A reaction 
 * specification is identified by its file name, which contains the actual
 * specifications and is used to invoke the reaction.
 * 
 * The file name is mapped to an INT, which is its handle.  The handle
 * is generated by the compiler and is used by the code to refer to a particular
 * reaction specification.  It is set using the setHandle(int handle) method.
 * 
 * The CodeWriter is responsible for assigning the handle to each reaction,
 * and treating the reaction specifications as global variables.
 * 
 * @author Chien-Liang Fok
 *
 */
public class RxnDecl  {
	private String fileName;
	private int handle = -1, lineno, charno;
	
	public RxnDecl(String fileName, int lineno, int charno) {
		this.fileName = fileName;
		this.lineno = lineno;
		this.charno = charno;
	}
	
	/**
	 * Determines whether the reaction declaration's handle has been set.
	 * 
	 * @return whether the reaction declaration's handle has been set.
	 */
	public boolean hasHandle() { return handle != -1; }
	
	/**
	 * Returns the reaction declaration's handle.
	 * 
	 * @return the reaction declaration's handle.
	 */
	public int handle() { return handle; }
	
	/**
	 * Returns the name of the reaction specification.  This is also the name of the
	 * reference to the specification using in bind and invoke operations.
	 * 
	 * @return the name of the reaction specification.
	 */
	public String name() { return fileName; }
	
	/**
	 * Converts the reaction decl into a reaction spec.
	 * 
	 * @return The reaction spec named by this reaction decl.
	 * @throws Exception
	 */
// TODO: Implement the reaction compiler!
//	public Rxn getRxn() throws Exception {
//		return new RxnC().compile(fileName);  // TODO: Make use of lineno and charno here.
//	}
	
	/**
	 * Sets the reaction declaration's handle.
	 * 
	 * @param handle the reaction declaration's handle.
	 * @throws AssemblerException If the handle is not valid.
	 */
	public void setHandle(int handle) { 
		this.handle = handle; 
	}
	
	/**
	 * Returns a string representation of this class.
	 * 
	 * @return a string representation of this class.
	 */
	public String toString() { return "(RxnDecl "+fileName+" ("+handle+"))"; }
	
}
