package servilla.compiler.tree;

import java.util.*;

/**
 * A sends data to the base station.
 * 
 * @author Chien-Liang Fok
 */
public class SendExpr extends Expr {
	private Vector<Expr> params;

//	private Hashtable<String, BuiltInMethod> builtInMethodTable = new Hashtable<String, BuiltInMethod>();
	
	public SendExpr(Vector<Expr> params, int lineno, int charno) {
		super(lineno, charno);
		this.params = params;
	}
	
	/**
	 * The resulting stack state must match that specified by OPsendM.nc.
	 */
	public void assemble(CodeWriter cw) throws AssemblerException {
		Enumeration<Expr> e = params.elements();
		while (e.hasMoreElements()) {
			Expr currExpr = e.nextElement();
			currExpr.assemble(cw);  // assemble the expression that generates the value
			Type type = currExpr.retType(cw);
			for (int i = 0; i < type.numDim(); i++) {
				cw.add(new Instruction(ISA.push2, lineno, charno).setParam2(type.dimSize(i)));
			}
			cw.add(new Instruction(ISA.push2, lineno, charno).setParam2(type.numDim()));
			cw.add(new Instruction(ISA.push2, lineno, charno).setParam2(type.type().value()));
		}
		cw.add(new Instruction(ISA.push4, lineno, charno).setParam4(params.size()));
		cw.add(new Instruction(ISA.send, lineno, charno));
	}
	
	public String toString() {
		StringBuffer pList = new StringBuffer();
		Enumeration e = params.elements();
		while (e.hasMoreElements()) {
			pList.append(e.nextElement());
			if (e.hasMoreElements())
				pList.append(", ");
		}
		return "(SendExpr: " + pList;
	}

	@Override
	public long initAddr(long startAddr) {
		Enumeration<Expr> e = params.elements();
		while (e.hasMoreElements()) {
			Expr s = e.nextElement();
			startAddr = s.initAddr(startAddr);
		}
		return startAddr;
	}


	@Override
	public Type retType(CodeWriter cw) throws AssemblerException { return Type.VOID; }


	@Override
	public int retHeapSize(CodeWriter cw) throws AssemblerException { return retType(cw).heapSize(); }
	
	@Override
	public int retStackSize(CodeWriter cw) throws AssemblerException { return retType(cw).stackSize(); }
	
	@Override
	public long localVarSize() {
		long result = 0;
		Enumeration<Expr> pv = params.elements();
		while (pv.hasMoreElements()) {
			result += pv.nextElement().localVarSize();
		}
		return result;
	}
}
