package servilla.compiler.tree;

/**
 * A send statement is used to send data to the base station.  It consists of
 * a send expression.
 * 
 * @author Chien-Liang Fok
 */
public class SendStatement extends Statement {
	private SendExpr expr;
	
	public SendStatement(SendExpr expr, int lineno, int charno) {
		super(lineno, charno);
		this.expr = expr;
	}
	
	public void assemble(CodeWriter cw) throws AssemblerException {
		expr.assemble(cw);
		popRetVal(cw, expr, "Discard result of send expression");
	}
	
	public String toString() {
		return "(SEND " + expr + ")";
	}

	@Override
	public long initAddr(long startAddr) {
		return expr.initAddr(startAddr);
	}
	
	@Override
	public long localVarSize() {
		return expr.localVarSize();
	}
}
