package servilla.compiler.tree;

import servilla.service.tree.DataType;

public class StringDeclExpr extends VarDeclExpr {
	/**
	 * The number of bytes allocated to the string.
	 */
	private int size;
	
	public StringDeclExpr(String name, DataType type, int lineno, int charno) {
		this(name, type, null, lineno, charno);
	}
	
	public StringDeclExpr(String name, DataType type, Expr initVal, int lineno, int charno) {
		super(name, type, initVal, lineno, charno);
	}
	
	/**
	 * Returns the number of bytes needed by local variables within the expression.
	 */
	@Override
	public long localVarSize() { return size; }
	
	/**
	 * Assembles the int decl expression. Adds itself to the CodeWriter's list of 
	 * local variables.  If there is an initial value, it creates an assignment 
	 * statement to initialize it.
	 */
	public void assemble(CodeWriter cw) throws AssemblerException {
		cw.addLocalVar(this, lineno, charno);
		Expr initValExpr = initVal();
		if (initValExpr != null) {
			DataType initType = initValExpr.retType(cw);
			if (initType == DataType.INT)
				initVar(cw, new VarRef(name(), lineno, charno), initValExpr);
			else
				throw new AssemblerException("Type mismatch, got " + initType + ", expected " + DataType.INT, lineno, charno);
		}
	}
}
