package servilla.compiler.tree;

/**
 * A unary statement.  Ex: i++;
 * 
 * @author Chien-Liang Fok
 *
 */
public class UnaryStatement extends Statement {
	private UnaryExpr expr;
	
	public UnaryStatement(UnaryExpr expr, int lineno, int charno) {
		super(lineno, charno);
		this.expr = expr;
	}
	
	/**
	 * Unary statements must pop the extra result on the stack.
	 */
	public void assemble(CodeWriter cw) throws AssemblerException {
		expr.assemble(cw);
		popRetVal(cw, expr, "Discard result of unary expression");
	}
	
	public String toString() {
		return "(UnaryStatement: " + expr + ")";
	}

	@Override
	public long initAddr(long startAddr) {
		return expr.initAddr(startAddr);
	}
	
	@Override
	public long localVarSize() {
		return expr.localVarSize();
	}
}
