package servilla.compiler.tree;

/**
 * Holds the name of a declared variable and the
 * expression that generates its initial value.
 * 
 * @author Chien-Liang Fok
 *
 */
public class VarDeclNameVal {
	private int lineno, charno;
	private Expr initVal = null; // default no initial value
	private String name;
	
	/**
	 * A constructor for a value with no value.
	 * 
	 * @param name The name of the variable.
	 * @param lineno The line number.
	 * @param charno The character number.
	 */
	public VarDeclNameVal(String name, int lineno, int charno) {
		this(name, null, lineno, charno);
	}
	
	/**
	 * A constructor for a value with an initial value.
	 * 
	 * @param name The name of the variable.
	 * @param initVal An expression generating the variable's initial value.
	 * @param lineno The line number.
	 * @param charno The character number.
	 */
	public VarDeclNameVal(String name, Expr initVal, int lineno, int charno) {
		this.name = name;
		this.initVal = initVal;
		this.lineno = lineno;
		this.charno = charno;
	}
	
	public String name() { return name; }
	public Expr initVal() { return initVal; }
	public int lineno() { return lineno; }
	public int charno() { return charno; }
}
