package servilla.compiler.tree;

/**
 * A VarDeclStmt is a VarDeclExpr followed by a semicolon.
 * 
 * @author Chien-Liang Fok
 */
public class VarDeclStmt extends Statement {
	private VarDeclExpr e;
	
	public VarDeclStmt(VarDeclExpr expr, int lineno, int charno) {
		super(lineno, charno);
		this.e = expr;
	}
	
	/**
	 * The VarDeclStatement does not need to pop anything off the stack since
	 * the VarDeclExpr does not push anything onto the stack.
	 */
	public void assemble(CodeWriter cw) throws AssemblerException { e.assemble(cw); }
	
	public String toString() { return "(VarDeclStatement: " + e + ")"; }

	@Override
	public long initAddr(long startAddr) { return e.initAddr(startAddr); }

	@Override
	public long localVarSize() { return e.localVarSize(); }
}
