package servilla.msgs.reliable;

import java.util.*;
import servilla.Script;
import servilla.service.tree.*;
import servilla.msgs.ServillaSpecMsg;
import servilla.util.*;
import org.apache.commons.lang.ArrayUtils;

/**
 * This class encapsulates a ServillaSpecMsg.  It contains the service
 * specifications belonging to a script.
 * 
 * @author Chien-Liang Fok
 */
public class ServillaSpecMsgJ {

	/**
	 * The script whose specifications will be included in this message.
	 */
	private Script s; 
	
	/**
	 * Create a ServillaSpecMsg.
	 *  
	 * @param s The script that contains the specs.
	 */
	public ServillaSpecMsgJ(Script s) {this.s = s;}
	
	public short amType() { return ServillaSpecMsg.AM_TYPE; }
	
	/**
	 * Converts this ServillaSpecMsgJ into an array of bytes.  The first
	 * two bytes is the script's ID.  The remaining bytes are the specifications.
	 * 
	 * @return The byte representation of the ServillaSpecMsgJ.
	 */
	public byte[] toByteArray() {
		Vector<Byte> bv = new Vector<Byte>();
		ByteUtils.saveBytes2(bv, s.id()); // The first two bytes is script ID
		byte[] byteCode;
		try {
			byteCode = s.specBytes();
		} catch(SpecCompilerException e) {
			e.printStackTrace();
			return null;
		}
		bv.addAll(Arrays.asList(ArrayUtils.toObject(byteCode)));
		return ArrayUtils.toPrimitive(bv.toArray(new Byte[]{}));
	}
	
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("ServillaSpecMsg: id="+s.id() + "\n");
		byte[] byteCode;
		try {
			byteCode = s.specBytes();
		} catch(SpecCompilerException e) {
			e.printStackTrace();
			return "Error converting ServillaSpecMsg to string";
		}
		result.append(ArrayToHexString.toString(byteCode));
		return result.toString();
	}
}
