// $Id: MessageJ.java,v 1.1 2005/10/13 17:12:19 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2007, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
package servilla.network;

import java.util.*;
import servilla.util.*;

import org.apache.commons.lang.ArrayUtils;

/**
 * This is the data packet that is used with the reliable network interface.
 * It assembles the fragments of data contained in a collection of RMsg messages.
 * 
 * @author Chien-Liang Fok
 *
 */
public class DataMsg {
	private int dest;
	private int am;
	Vector<Short> data;
	
	/**
	 * Creates a data packet.
	 * 
	 * @param dest The destination address
	 * @param am The message type.
	 * @param data The data to send.
	 */
	public DataMsg(int dest, int am, byte[] data) {
		this.dest = dest;
		this.am = am;
		this.data = new Vector<Short>();
		Short[] sa = ShortByteUtils.swapType(ArrayUtils.toObject(data));
		this.data.addAll(Arrays.asList(sa));
	}
	
	/**
	 * Creates a data packet.
	 * 
	 * @param dest The destination address
	 * @param am The message type.
	 * @param data The data in the data packet.
	 */
	public DataMsg(RMsgJ rm) {
		this.dest = rm.replyAddr();
		this.am = rm.am();
		this.data = new Vector<Short>((int)rm.size());
		saveMsg(rm);
	}
	
	/**
	 * Saves the data within a reliable message in this DataMsg.
	 * A DataMsg is broken down in to multiple ReliableMsgs during
	 * wireless transmission, then reassembled at the destination.
	 * @param rm
	 */
	public void saveMsg(RMsgJ rm) {
		data.addAll((int)rm.startAddr(), rm.data());
	}
	/**
	 * Returns the packet's destination.
	 * @return the packet's destination.
	 */
	public int dest() { return dest; }
	
	/**
	 * Returns the active message type.
	 * @return the active message type.
	 */
	public int am() { return am; }
	
	/**
	 * Returns the number of bytes in the data packet.
	 * 
	 * @return the data size.
	 */
	public int numBytes() { return data.size(); }
	
	/**
	 * Returns a value in the data.
	 * 
	 * @param indx The address of the value, must be between 0 and dataSize()-1.
	 * @return The value at the specified address.
	 */
	public short data(int indx) { return data.get(indx).shortValue(); }
	
	/**
	 * Returns all of the data stored in this message as an array of bytes.
	 * 
	 * @return all of the data stored in this message as an array of bytes.
	 */
	public byte[] data() {
		return ArrayUtils.toPrimitive(ShortByteUtils.swapType(data.toArray(new Short[]{})));
	}
	
	/**
	 * Returns a string representation of this class.
	 */
	public String toString() {
		return "DataMsg: dest=" + dest + " am="+ArrayToHexString.intToHex(am) + " data=[" + 
			ArrayToHexString.toString(ArrayUtils.toPrimitive(data.toArray(new Short[]{}))) + "]";
	}
	
}
