package servilla.network;

/**
 * Provides a reliable message delivery interface for
 * sending and receving messages to and from the sensor
 * network.
 * 
 * @author Chien-Liang Fok
 */
public class RSNI {
	private RMsgSndr sender;
	private RMsgRcvr rcvr;
	
	/**
	 * The constructor.
	 * 
	 * @param sni The sensor network interface.
	 */
	public RSNI(SNI sni) {
		sender = new RMsgSndr(sni);
		rcvr = new RMsgRcvr(sni);
	}
	
	/**
	 * Sends data to a node in the sensor network.
	 * 
	 * @param dest The destination address.
	 * @param id The message ID.
	 * @param data The data to send.
	 * @return true if successful, false otherwise.
	 */
	public boolean send(int dest, short id, byte[] data) {
		return sender.send(dest, id, data);
	}
	
	/**
	 * Registers a listener for reliable data messages.
	 * 
	 * @param id The message ID.
	 * @param rcvr The object implementing the RMsgListener interface.
	 */
	public void registerListener(int amType, RMsgListener rcvr) {
		this.rcvr.addMsgListener(amType, rcvr);
	}
	
	/**
	 * Deregisters a listener.
	 * 
	 * @param id The AM type of the message being listened for.
	 * @param rcvr The object that was listening to the message.
	 */
	public void deregisterListener(int amType, RMsgListener rcvr) {
		this.rcvr.removeMsgListener(amType, rcvr);
	}
}
