package servilla.service.tree;

import java.util.*;

/**
 * An attribute property has a type of PropertyType.ATTRIBUTE and three
 * values: a string (the name of the attribute), a relation (an operator
 * on the attribute's value, and a value (the value of the attribute).
 *  
 * @author Chien-Liang Fok
 *
 */
public class AttributeProperty implements Property {
	StringValue name;
	Relation r;
	Value v;
	
	public AttributeProperty(StringValue name, Relation r, Value v) {
		this.name = name;
		this.r = r;
		this.v = v;
	}
	
	public PropertyType type() {
		return PropertyType.ATTRIBUTE;
	}
	
	public Relation relation() {
		return r;
	}
	
	public Value value() {
		 return v;
	}
	
	public Vector<Byte> toByteVector() {
		Vector<Byte> bv = new Vector<Byte>();
		
//		Attributes do not need to specify their property type since they 
//		are always located at the end
//		bv.add(PropertyType.ATTRIBUTE.value()); 
		
		bv.addAll(name.toByteVector());
		
		bv.add(r.value());
		bv.addAll(v.toByteVector());
		
		return bv;
	}
	
	public String toString() {
		return type() + " " + name + " " + r.mathSymbol() + " " + v; 
	}

	public static void main(String[] args) {
		AttributeProperty ap = new AttributeProperty(new StringValue("Reliability"),
				Relation.GT, new FloatValue((float)0.99));
		System.out.println(ap.toString());
	}
}
