package servilla.service.tree;

import servilla.util.ArrayToHexString;

import java.util.*;

/**
 * A float value.
 *
 * @author Chien-Liang Fok
 */
public class FloatValue extends Value {
	float value;
	
	public FloatValue(float value) { this.value = value; }
	public float value() { return value; }
	
	/**
	 * Returns the type of the value.
	 * 
	 * @return the type of the value.
	 */
	public DataType type() { return DataType.FLOAT32; }
	
	/**
	 * Convert the float value into a vector of bytes.
	 * Note that the float value is stored in little-endian format.
	 * 
	 * This is the same format as the MSP430 processor.
	 * 
	 * Byte format: [data type (1B)] [float value (4B)]
	 * 
	 * @see service.Value#toByteArray()
	 * @return the byte representation of this class.
	 */
	public Vector<Byte> toByteVector() {
		Vector<Byte> bv = new Vector<Byte>();
		bv.add(DataType.FLOAT32.value());
		int i = Float.floatToRawIntBits(value); 
		for (int j = 0; j < 4; j++) {
			bv.add((byte)(i >> j*8));
		}
		return bv;
	}
	
	/**
	 * Prints a string representation of this component.
	 */
	public String toString() {
		return ""+value;
	}
	
	/**
	 * Tests this component.
	 * 
	 * @param args Input arguments (unused)
	 */
	public static void main(String[] args) {
		System.out.println("FloatValue: 0.25");
		FloatValue rfv1 = new FloatValue((float)0.25);
		System.out.println(ArrayToHexString.toString(rfv1.toByteArray()) + "\n");
		
		System.out.println("FloatValue: 0.5");
		rfv1 = new FloatValue((float)0.50);
		System.out.println(ArrayToHexString.toString(rfv1.toByteArray()) + "\n");	
		
		System.out.println("FloatValue: 3.5");
		rfv1 = new FloatValue((float)3.50);
		System.out.println(ArrayToHexString.toString(rfv1.toByteArray()) + "\n");	
	}
}