package servilla.service.tree;

import java.util.*;

/**
 * An input property has a type of PropertyType.INPUT and a tuple
 * value (the property's interface).
 *  
 * @author Chien-Liang Fok
 *
 */
public class InputProperty implements Property {
	Tuple t;
	
	/**
	 * Creates n InputProperty.  
	 * 
	 * @param t The tuple defining the input interface of the service.
	 */
	public InputProperty(Tuple t) { this.t = t; }
	
	/**
	 * Returns the input interface of the service.
	 * 
	 * @return the input interface of the service.
	 */
	public Tuple value() { return t; }
	
	/**
	 * Returns the property type.
	 */
	public PropertyType type() { return PropertyType.INPUT; }
	
	/**
	 * Convert the InputProperty into a vector of bytes.
	 * 
	 * @see service.Property#toByteVector()
	 * @return the byte representation of this class.
	 */
	public Vector<Byte> toByteVector() {
		Vector<Byte> bv = new Vector<Byte>();
		//bv.add(type().value());  The input property is always the second property
		bv.addAll(t.toByteVector());
		return bv;
	}
	
	/**
	 * Returns the string representation of this class.
	 */
	public String toString() { return type() + " = " + value(); }
}
