package servilla.service.tree;

import java.util.*;

/**
 * A name property has a type of PropertyType.NAME and a string
 * value (the name of the property).
 *  
 * @author Chien-Liang Fok
 *
 */
public class NameProperty implements Property {
	StringValue name;
	
	/**
	 * Creates a NameProperty.  
	 * 
	 * @param name The name of the service.
	 */
	public NameProperty(StringValue name) { 
		this.name = name;
	}
	
	/**
	 * Returns the name of the service.
	 * 
	 * @return the name of the service.
	 */
	public StringValue name() { return name; }
	
	/**
	 * Returns the property type.
	 */
	public PropertyType type() {
		return PropertyType.NAME;
	}
	
	/**
	 * Convert the NameProperty into a vector of bytes.
	 * The standard ASCII byte conversion is used.  The last byte
	 * is a null (0x00) value to indicate the end of the string.
	 * 
	 * Byte format: [StringValue (nB)]
	 * 
	 * @see service.Value#toByteVector()
	 * @return the byte representation of this class.
	 */
	public Vector<Byte> toByteVector() {
		Vector<Byte> bv = new Vector<Byte>();
//		The name property does not need to specify the data type since it
//		is always a String.
//		bv.add(type().value());
		bv.addAll(name.toByteVector());
		return bv;
	}
	
	/**
	 * Returns the string representation of this class.
	 */
	public String toString() { return type() + " = " + name(); }
}
