package servilla.service.tree;

/**
 * Defines the valid relations supported by Servilla.
 * The values of these relations must match those within 
 * Servilla/types/Service.h
 * 
 * @author Chien-Liang Fok
 */
public enum Relation {
	EQ, 
	LT,
	GT,
	LTE,
	GTE;
	
	private byte value;
	
	static byte counter = 0;
	
	static byte nextCount() {
		return (byte)counter++;
	}
	
	Relation() {
		this.value = Relation.nextCount();
	}
	
	/**
	 *  Returns the byte representation of the relation.
	 */
	public byte value() {
		return value;
	}
	
	/**
	 *  Returns the math symbol representation of the relation.
	 */
	public String mathSymbol() {
		if (value == EQ.value())
			return "=";
		else if (value == LT.value())
			return "<";
		else if (value == GT.value())
			return ">";
		else if (value == LTE.value())
			return "<=";
		else if (value == GTE.value())
			return ">=";
		else 
			return "UNKNOWN RELATION"; // error condition
	}
	
	public static void main(String[] args) {

		Relation[] v = Relation.values();
		System.out.println("// Copy this into types/Service.h");
		System.out.println("// This was generated by servilla.service.tree.Relation");
		System.out.println("typedef enum {");
		for (int i = 0; i < v.length; i++) {
			System.out.println("\t" + v[i] + " = "+ v[i].value() + ",");
		}
		System.out.println("} Relation_t;\n");	
			
		System.out.println("// Copy this into the regexp section of servilla.service.ServiceSpec.lex");
		for (int i = 0; i < v.length; i++) {
			String toPrint = v[i] + "=\"" + v[i].mathSymbol() + "\"";
			if (i == 0) toPrint += "// This was generated by servilla.service.tree.Relation";
			System.out.println(toPrint);
		}
		System.out.println("");
			
		System.out.println("// Copy this into the production section of servilla.service.ServiceSpec.lex");
		System.out.println("// This was generated by servilla.service.tree.Relation");
		for (int i = 0; i < v.length; i++) {
			System.out.println("<YYINITIAL> {" + v[i] + "} { return new Symbol(sym." + v[i] + ", new StringToken(yytext(), yyline, yychar)); }");
		}
		System.out.println("");
		
		System.out.println("// Copy this into the production section of servilla.service.ServiceSpec.cup");
		System.out.println("// This was generated by servilla.service.tree.Relation");
		StringBuffer sb = new StringBuffer();
		sb.append("relation ::=\n");
		for (int i = 0; i < v.length; i++) {
			sb.append("\t" + v[i] + " {: RESULT= Relation." + v[i] + "; :}");
			if (i < v.length-1) sb.append(" | \n");
		}
		System.out.println(sb + ";\n");
	}
}
