package servilla.service.tree;

import java.util.*;

/**
 * Maps the name of a service to its byte representation.
 * Each service name must have a unique byte representation.
 * Thus, there can be at most 257 services.
 * 
 * The service name is unique to each agent.  It is used to
 * refer to a service within the agent's code.
 * 
 * @deprecated Use the CodeWriter component to keep track of
 * what number each service is assigned to.
 * 
 * @see service.compiler.tree.CodeWriter
 * 
 * @author Chien-Liang Fok <liangfok@wustl.edu>
 */
public class ServiceSpecNameTable {
	private static final ServiceSpecNameTable nameTable = new ServiceSpecNameTable();  // singleton
	private Hashtable<String, Byte> table;
	private static byte counter = 0;
	
	private ServiceSpecNameTable() {
		table = new Hashtable<String, Byte>();
	}
	
	public static final ServiceSpecNameTable getTable() {
		return nameTable;
	}
	/**
	 * Adds a service name to the table.  Ensure it is mapped
	 * to a unique byte.
	 * 
	 * @param name
	 * @return
	 */
	public byte addName(String name) {
		table.put(name, counter++);
		return table.get(name);
	}
	
	/**
	 * Returns the byte representation of the service specification's
	 * name.  If the name is not already in the table, add it to the
	 * table.
	 * 
	 * @param name The name of the service.
	 * @return The byte representation of the service name.
	 */
	public byte get(String name) {
		if (table.containsKey(name))
			return table.get(name);
		else
			return addName(name);
	}

}
