package servilla.service.tree;

import java.util.*;
import org.apache.commons.lang.ArrayUtils;

import servilla.util.ArrayToHexString;

/**
 * A string value.
 *
 * @author Chien-Liang Fok
 */
public class StringValue extends Value {
	String value;
	
	public StringValue(String value) {
		this.value = value;
		//if (toByteArray().length > 100)
			//throw new SpecCompilerException("Attempting to create StringValue that exceeds max size");
	}
	
	public String value() {
		return value;
	}
	
	/**
	 * Returns the type of the value.
	 * 
	 * @return the type of the value.
	 */
	public DataType type() {
		return DataType.STRING;
	}
	
	/**
	 * Convert the StringValue into a vector of bytes.
	 * The standard ASCII byte conversion is used.  The last byte
	 * is a null value to indicate the end of the string.
	 * 
	 * Byte format: [string (nB)] [0x00 (1B)]
	 * 
	 * @see service.Value#toByteVector()
	 * @return the byte representation of this class.
	 */
	public Vector<Byte> toByteVector() {
		Vector<Byte> bv = new Vector<Byte>();
		//bv.add(DataType.STRING.value()); // taken away because property NAME does not specify the type.
		
		Byte[] nameBytes = ArrayUtils.toObject(value.getBytes());
		bv.addAll(Arrays.asList(nameBytes));
		
		bv.add((byte)0x00); // null character at end
		return bv;
	}
	
	/**
	 * Returns the string representation of this class.
	 */
	public String toString() { return value; }
	
	public static void main(String[] args) throws Exception {
		String s = "abc";
		System.out.println("StringValue: = \"" + s + "\"");
		StringValue sv1 = new StringValue(s);
		System.out.println(ArrayToHexString.toString(sv1.toByteArray()) + "\n");
	}
}