package servilla.util;

import java.util.*;

/**
 * Converts an array of bytes or shorts into a hex string.
 * 
 * @author Chien-Liang Fok
 */
public class ArrayToHexString {

	/**
	 * Translate a byte into its hexadecimal representation.
	 * For example, byte 1 will be trasnlated into "0x01".
	 * 
	 * @param b The byte to translate.
	 * @return The string representation of the byte.
	 */
	public static String byteToHexString(byte b) {
		//String result = Long.toHexString(b & 0xff);
		String result = Integer.toHexString(0xff & b);
		if (result.length() == 1)
			result = "0" + result;
		result = "0x"+result;
		return result;
	}
	
	/**
	 * Translate a byte into its hexadecimal representation.
	 * For example, byte 1 will be trasnlated into "0x01".
	 * 
	 * @param b The byte to translate.
	 * @return The string representation of the byte.
	 */
	public static String toString(Vector<Byte> bv) {
		return toString(org.apache.commons.lang.ArrayUtils.toPrimitive(bv.toArray(new Byte[]{})));
	}
	
	/**
	 * Converts an array of bytes into a string that contains the
	 * textual representation of each byte separated by spaces.
	 * For example, byte array {1, 2} will be translated into
	 * "0x01, 0x02".
	 * 
	 * @param ba The array of bytes.
	 * @return The string representation of the byte array.
	 */
	public static String toString(byte[] ba) {
		StringBuffer result = new StringBuffer();
		for (int i = 0; i < ba.length; i++) {
			if (i == 0)
				result.append(byteToHexString(ba[i]));
			else
				result.append(" " + byteToHexString(ba[i]));
		}
		return result.toString();
	}
	
	public static String shortToHex(short b) {
		//String result = Long.toHexString(b & 0xff);
		String result = Integer.toHexString(0xff & b);
		if (result.length() == 1)
			result = "0" + result;
		result = "0x"+result;
		return result;
	}
	
	public static String intToHex(int i) {
		//String result = Long.toHexString(b & 0xff);
		String result = Integer.toHexString(0xff & i);
		if (result.length() == 1)
			result = "0" + result;
		result = "0x"+result;
		return result;
	}
	
	public static String toString(short[] ba) {
		StringBuffer result = new StringBuffer();
		for (int i = 0; i < ba.length; i++) {
			if (i == 0)
				result.append(shortToHex(ba[i]));
			else
				result.append(" " + shortToHex(ba[i]));
		}
		return result.toString();
	}
}
