// $Id: Debugger.java,v 1.1 2005/10/13 17:12:19 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * Debugger.java
 *
 * @author Chien-Liang Fok
 */

package servilla.util;

public class Debugger {
	public static boolean debug = false;
	
	/**
	 * Prints a message to stderr.
	 * @param header The message's header, usually the component that is calling this.
	 * @param msg The actual message to print.
	 */
	public static void err(String header, String msg) { System.err.println(header + ": " + msg); }
	
	/**
	 * Prints a message to stdout if in debug mode.
	 * @param header The message's header, usually the component that is calling this.
	 * @param msg The actual message to print.
	 */
	public static void dbg(String header, String msg) { if (debug) log(header, msg); }
	
	/**
	 * Prints a message to stdout.
	 * @param header The message's header, usually the component that is calling this.
	 * @param msg The actual message to print.
	 */
	public static void log(String header, String msg) {
		if (header != null) 
			System.out.println(header + ": " + msg);
		else 
			System.out.println(msg);
		System.out.flush();
	}
	
	/**
	 * Prints a warning message to stdout.  A warning message is a regular message
	 * except with the string "WARNING" in front of it.
	 * 
	 * @param header The message's header, usually the component that is calling this.
	 * @param msg The actual message to print.
	 */
	public static void warn(String header, String msg) { log("WARNING: " + header, msg); }
}

