// $Id: TimeoutTimer.java,v 1.2 2005/11/10 04:33:36 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * TimeoutTimer.java
 *
 * @author Chien-Liang Fok
 */

package servilla.util;

import servilla.ServillaConstants;

/**
 * A timer for timing out various events in Servilla.
 * 
 * @author Chien-Liang Fok
 *
 */
public class TimeoutTimer implements ServillaConstants, Runnable {
	
	private boolean alive = true, killed = false;
	private Object lock;
	private Thread thread;
	private long duration;
	
	/**
	 * Creates a new TimeoutTimer.
	 * 
	 * @param lock the lock to notify when the timer fires.
	 * @param duration the timeout in ms
	 */
	public TimeoutTimer(Object lock, long duration) {
		this.lock = lock;
		this.duration = duration;
		thread = new Thread(this);
		thread.start();
		//Debugger.dbg("TimoutTimer", "Starting TimeoutTimer, duration " + duration);
	}
	
	public void kill() {
		synchronized(lock) {
			alive = false;
			killed = true;
		}
		try {
			thread.interrupt();
			//thread.join();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void run() {
		try {
			Thread.sleep(duration);
		} catch(InterruptedException e) {
			if (!killed) e.printStackTrace();
		}
		//Debugger.dbg("TimeoutTimer", "TIMER FIRED!");
		
		if (alive) {
			//Debugger.dbg("TimeoutTimer", "TIMER IS ALIVE!");
			synchronized(lock) {
				if (!killed) {
				  Debugger.dbg("TimoutTimer", "TIMED OUT!");
				  try { lock.notifyAll(); } catch(Exception e) { e.printStackTrace(); }
				}
			}
		}// else Debugger.dbg("TimoutTimer", "TIMER FIRED BUT WAS NOT ALIVE!");
	}
}


