// $Id: ServillaValue.java,v 1.1 2005/10/13 17:12:19 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * ServillaSVInvalid.java
 *
 * @author Chien-Liang Fok
 */

package servilla.variables;
import servilla.*;
import servilla.service.tree.DataType;

public class Float32Var extends ServillaVariable implements ServillaConstants, java.io.Serializable 
{

	private static final long serialVersionUID = 858859045904934699L;
	private float value;
	
	public Float32Var(float value) {
		this((short)0, value);
	}
	
	public Float32Var(short handle, float value) {
		super(handle);
		this.value = value;
	}
	
	public ServillaVariable deepClone() {
		return new Float32Var(handle, value);
	}
	
	public DataType getType() {
		return DataType.INT16;
	}
	
	/**
	 * Converts the float into 4 bytes based on
	 * IEEE's 754 standard.
	 */
	public short getSize() {
		return 4;
	}
	
	public short[] toBytes() {
		short[] result = new short[4];
		int i = Float.floatToRawIntBits(value); 
		result[0] = (short)(i & 0xff);
		result[1] = (short)(i >> 8 & 0xff);
		result[2] = (short)(i >> 16 & 0xff);
		result[3] = (short)(i >> 24 & 0xff);
		return result;
	}
	
	public float getValue() {
		return value;
	}
	
	public boolean matches(ServillaVariable v) {
		if (v instanceof Float32Var) {
			Float32Var iv = (Float32Var)v;
			return iv.getValue() == getValue();
		} else return false;
	}
	
	/*public ServillaValue xor(ServillaString key) {
		return new ServillaValue((short)(value ^ key.getString()));
	}*/
	
	public boolean equals(Object v) {
		if (v instanceof Float32Var) {
			Float32Var as = (Float32Var)v;
			return as.getValue() == getValue();
		} else
			return false;
	}
	
	
	public String toString() {
		return "[Float32Var: " + value + "]";
	}
}

