// $Id: ServillaString.java,v 1.2 2005/12/09 07:24:06 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * ServillaSVInvalid.java
 *
 * @author Chien-Liang Fok
 */

package servilla.variables;
import servilla.*;
import servilla.service.tree.DataType;

public class StringVar extends ServillaVariable implements ServillaConstants,
	java.io.Serializable
{
	private static final long serialVersionUID = 3943180327559295278L;
	private String string;
	
	public StringVar(String var) {
		this((short)0, var);
	}
	public StringVar(short handle, String var) {
		super(handle);
		this.string = var;
	}
	
	
	public ServillaVariable deepClone() {
		return new StringVar(handle, string);
	}
	
	/**
	 * An ServillaString is two bytes.
	 */
	public String getString() {
		return string;
	}
	
	public short getSize() {
		return (short)(string.length() + 1);
	}
	
	/**
	 * Returns the byte representation of the StringVar.
	 * 
	 * Format: [char 1][char 2]...[char n] [0x00]
	 */
	public short[] toBytes() {
		byte[] charBytes = string.getBytes();
		short[] result = new short[charBytes.length+1];
	
		result[result.length-1] = 0x00; // null character at end
		for (int j = 0; j < charBytes.length; j++) {
			result[j] = charBytes[j];
		}
		return result;
	}
	
	public DataType getType() {
		return DataType.STRING;
	}
	
	public boolean matches(ServillaVariable v) {
		if (v instanceof StringVar) {
			StringVar an = (StringVar)v;
			return an.getString().equals(getString());
		} else return false;
	}
	
	public boolean equals(Object v) {
		if (v instanceof StringVar) {
			StringVar as = (StringVar)v;
			return as.string.equals(string);
		} else
			return false;
	}
	
	public String toString() {
		return "[StringVar: " + string + "]";
	}
}

