// $Id: ServillaType.java,v 1.1 2005/10/13 17:12:19 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * ServillaSVInvalid.java
 *
 * @author Chien-Liang Fok
 */

package servilla.variables;
import servilla.*;
import servilla.service.tree.DataType;

public class TypeVar extends ServillaVariable implements ServillaConstants,
	java.io.Serializable
{
	private static final long serialVersionUID = -3602609265130960981L;
	private DataType dType;  // data type
	
	public TypeVar(DataType dType) {
		this((short)0, dType);
	}
	
	public TypeVar(short handle, DataType dType) {
		super(handle);
		this.dType = dType;
	}
	
	/**
	 * An ServillaType is two bytes.
	 */
	public short getSize() {
		return 2;
	}
	
	public short[] toBytes() {
		short[] result = new short[1];
		result[0] = dType.value();
		return result;
	}
	
	public ServillaVariable deepClone() {
		return new TypeVar(handle, dType);
	}
	
	/**
	 * Returns AGILLA_TYPE_TYPE.
	 */
	public DataType getType() {
		return DataType.TYPE;
	}
	
	public DataType getDataType() {
		return dType;
	}
	
	public boolean equals(Object o) {
		if (o instanceof TypeVar) {
			TypeVar t = (TypeVar)o;
			return t.getDataType() == getDataType();
		}
		return false;
	}
	
	public boolean matches(ServillaVariable v) {
		if(dType == DataType.ANY)
			return true;
		else if (dType == DataType.INT16)
			return v instanceof Int16Var;
		else if(dType == DataType.FLOAT32)
			return v instanceof Float32Var;
		else if (dType == DataType.STRING)
			return v instanceof StringVar;
		else if (dType == DataType.TYPE)
			if (v instanceof TypeVar) {
				return ((TypeVar)v).equals(this);
			} else return false;
		return false;
	}
	
	public String toString() {
		return "[Type Variable: data type = " + dType + "]";
	}
}

