// $Id: ServillaValue.java,v 1.1 2005/10/13 17:12:19 chien-liang Exp $

/* Servilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Servilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Servilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Servilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * scripts from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, scripts or third parties acting on behalf or
 * under authorization from you, as a result of using Servilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/**
 * ServillaSVInvalid.java
 *
 * @author Chien-Liang Fok
 */

package servilla.variables;
import servilla.service.tree.DataType;

public class Uint16Var extends ServillaVariable implements java.io.Serializable 
{

	private static final long serialVersionUID = 858859045904934699L;
	private short value;

	/**
	 * For use when value is stored on operand stack.
	 * 
	 * @param value
	 */
	public Uint16Var(short value) {
		this((short)0, value);
	}
	
	public Uint16Var(short handle, short value) {
		super(handle);
		this.value = value;
	}
	
	public ServillaVariable deepClone() {
		return new Uint16Var(handle, value);
	}
	
	public DataType getType() {
		return DataType.UINT16;
	}
	
	public short getSize() {
		return 2;
	}
	
	public short[] toBytes() {
		short[] result = new short[getSize()];
		result[0] = (short)(getValue() & 0xff);
		result[1] = (short)(getValue() >> 8 & 0xff);
		return result;
	}
	
	public short getValue() {
		return value;
	}
	
	public boolean matches(ServillaVariable v) {
		if (v instanceof Uint16Var) {
			Uint16Var iv = (Uint16Var)v;
			return iv.getValue() == getValue();
		} else return false;
	}
	
	/*public ServillaValue xor(ServillaString key) {
		return new ServillaValue((short)(value ^ key.getString()));
	}*/
	
	public boolean equals(Object v) {
		if (v instanceof Uint16Var) {
			Uint16Var as = (Uint16Var)v;
			return as.getValue() == getValue();
		} else
			return false;
	}
	
	
	public String toString() {
		return "[Uint16Var: " + value + "]";
	}
}

